@@ -3,10 +3,10 @@ from typing import Union
 
 import lab as B
 import wbml.out
+from lab.shape import Shape
 from plum import Dispatcher
 from wbml.warning import warn_upmodule
 
-from .shape import assert_matrix
 from .util import ToDenseWarning, dtype_str, indent
 
 __all__ = ["AbstractMatrix", "Dense", "repr_format", "structured"]
@@ -53,7 +53,7 @@ class AbstractMatrix(metaclass=abc.ABCMeta):
         return B.matmul(self, other)
 
     def __rmatmul__(self, other):
-        return B.matmul(other, self )
+        return B.matmul(other, self)
 
     def __getitem__(self, item):
         if structured(self):
@@ -113,17 +113,16 @@ class Dense(AbstractMatrix):
     """
 
     def __init__(self, mat):
-        assert_matrix(
-            mat,
-            "Input is not a rank-2 tensor. Can only construct "
-            "dense matrices from rank-2 tensors.",
-        )
-        self.mat = B.dense(mat)
+        self.mat = B.uprank(B.dense(mat))
         self.cholesky = None
 
     def __str__(self):
-        rows, cols = B.shape(self)
-        return f"<dense matrix: shape={rows}x{cols}, dtype={dtype_str(self)}>"
+        return (
+            f"<dense matrix:"
+            f" batch={Shape(*B.shape_batch(self))},"
+            f" shape={Shape(*B.shape_matrix(self))},"
+            f" dtype={dtype_str(self)}>"
+        )
 
     def __repr__(self):
         return (
