@@ -1,12 +1,15 @@
+from .abs import *
 from .add import *
 from .block import *
 from .block_diag import *
+from .broadcast_batch_to import *
 from .broadcast_to import *
 from .cholesky import *
 from .cholesky_solve import *
 from .concat import *
 from .dense import *
 from .diag import *
+from .diag_extract import *
 from .divide import *
 from .dtype import *
 from .eye import *
@@ -20,6 +23,7 @@ from .kron import *
 from .logdet import *
 from .matmul import *
 from .matmul_diag import *
+from .mean import *
 from .multiply import *
 from .negative import *
 from .one import *
@@ -34,6 +38,11 @@ from .root import *
 from .sample import *
 from .schur import *
 from .shape import *
+from .shape_batch import *
+from .shape_batch_broadcast import *
+from .shape_broadcast import *
+from .shape_matrix import *
+from .shape_matrix_broadcast import *
 from .solve import *
 from .sqrt import *
 from .squeeze import *
@@ -47,5 +56,6 @@ from .trace import *
 from .transpose import *
 from .triangular_solve import *
 from .uprank import *
+from .util import *
 from .zero import *
 from .zeros import *
