@@ -13,7 +13,9 @@ def fill_diag(a: B.Numeric, diag_len: B.Int):
         a (scalar): Scalar to fill diagonal with.
         diag_len (int): Length of diagonal.
     """
-    return Diagonal(a * B.ones(B.dtype(a), diag_len))
+    return Diagonal(
+        B.expand_dims(a, axis=-1) * B.ones(B.dtype(a), *B.shape(a), diag_len)
+    )
 
 
 B.fill_diag = fill_diag
