@@ -1,6 +1,7 @@
 import lab as B
 
 from matrix import Constant, Kronecker, Zero
+from matrix.ops.util import align_batch
 
 # noinspection PyUnresolvedReferences
 from ..util import (
@@ -27,14 +28,18 @@ from ..util import (
 )
 
 
+def _kron_matrix(a, b):
+    return B.kron(*align_batch(a, b), -2, -1)
+
+
 def test_kron_zero_diag(zero1, diag2):
-    check_bin_op(B.kron, zero1, diag2, asserted_type=Zero)
-    check_bin_op(B.kron, diag2, zero1, asserted_type=Zero)
+    check_bin_op(B.kron, zero1, diag2, ref=_kron_matrix, asserted_type=Zero)
+    check_bin_op(B.kron, diag2, zero1, ref=_kron_matrix, asserted_type=Zero)
 
 
 def test_kron_dense(dense1, dense2):
-    check_bin_op(B.kron, dense1, dense2, asserted_type=Kronecker)
+    check_bin_op(B.kron, dense1, dense2, ref=_kron_matrix, asserted_type=Kronecker)
 
 
 def test_kron_const(const1, const2):
-    check_bin_op(B.kron, const1, const2, asserted_type=Constant)
+    check_bin_op(B.kron, const1, const2, ref=_kron_matrix, asserted_type=Constant)
