@@ -1,5 +1,4 @@
 import lab as B
-import pytest
 
 from matrix import Diagonal
 
@@ -8,9 +7,13 @@ from .util import approx, dense1
 
 
 def test_diagonal_formatting():
-    assert str(Diagonal(B.ones(3))) == "<diagonal matrix: shape=3x3, dtype=float64>"
     assert (
-        repr(Diagonal(B.ones(3))) == "<diagonal matrix: shape=3x3, dtype=float64\n"
+        str(Diagonal(B.ones(3))) == ""
+        "<diagonal matrix: batch=(), shape=(3, 3), dtype=float64>"
+    )
+    assert (
+        repr(Diagonal(B.ones(3))) == ""
+        "<diagonal matrix: batch=(), shape=(3, 3), dtype=float64\n"
         " diag=[1. 1. 1.]>"
     )
 
@@ -21,9 +24,4 @@ def test_diagonal_attributes():
 
 
 def test_conversion_to_diagonal(dense1):
-    approx(Diagonal(dense1), B.diag(B.diag(dense1)))
-
-
-def test_diagonal_checks():
-    with pytest.raises(AssertionError):
-        Diagonal(B.ones(3, 3))
+    approx(Diagonal(dense1), B.diag_construct(B.diag_extract(dense1)))
