@@ -8,12 +8,12 @@ from .util import approx, dense1, dense2, diag1
 
 def test_lowertriangular_formatting():
     assert (
-        str(LowerTriangular(B.ones(3, 3)))
-        == "<lower-triangular matrix: shape=3x3, dtype=float64>"
+        str(LowerTriangular(B.ones(3, 3))) == ""
+        "<lower-triangular matrix: batch=(), shape=(3, 3), dtype=float64>"
     )
     assert (
-        repr(LowerTriangular(B.ones(3, 3)))
-        == "<lower-triangular matrix: shape=3x3, dtype=float64\n"
+        repr(LowerTriangular(B.ones(3, 3))) == ""
+        "<lower-triangular matrix: batch=(), shape=(3, 3), dtype=float64\n"
         " mat=[[1. 1. 1.]\n"
         "      [1. 1. 1.]\n"
         "      [1. 1. 1.]]>"
@@ -33,12 +33,12 @@ def test_conversion_to_lowertriangular(diag1):
 
 def test_uppertriangular_formatting():
     assert (
-        str(UpperTriangular(B.ones(3, 3)))
-        == "<upper-triangular matrix: shape=3x3, dtype=float64>"
+        str(UpperTriangular(B.ones(3, 3))) == ""
+        "<upper-triangular matrix: batch=(), shape=(3, 3), dtype=float64>"
     )
     assert (
-        repr(UpperTriangular(B.ones(3, 3)))
-        == "<upper-triangular matrix: shape=3x3, dtype=float64\n"
+        repr(UpperTriangular(B.ones(3, 3))) == ""
+        "<upper-triangular matrix: batch=(), shape=(3, 3), dtype=float64\n"
         " mat=[[1. 1. 1.]\n"
         "      [1. 1. 1.]\n"
         "      [1. 1. 1.]]>"
