@@ -1,7 +1,9 @@
 import re
 import warnings
+from itertools import product
 
 import lab as B
+import numpy as np
 import pytest
 from numpy.testing import assert_allclose
 from plum import Dispatcher, Union
@@ -18,6 +20,7 @@ from matrix import (
     Woodbury,
     Zero,
 )
+from matrix.ops.util import align_batch
 from matrix.util import ToDenseWarning
 
 __all__ = [
@@ -29,6 +32,7 @@ __all__ = [
     "ConditionalContext",
     "concat_warnings",
     # Fixtures:
+    "gen_batch_code",
     "mat1",
     "mat2",
     "vec1",
@@ -107,22 +111,40 @@ def _assert_instance(x, asserted_type):
     ), f"Expected instance of type {asserted_type} but got {type(x)}."
 
 
-def check_un_op(op, x, asserted_type=object):
+def check_un_op(op, x, ref=None, asserted_type=object):
     """Assert the correct of a unary operation by checking whether the
     result is the same on the dense version of the argument.
 
     Args:
         op (function): Unary operation to check.
+        ref (function, optional): Set this as the function to be used as the reference
+            to which the dense matrices will be fed.
         x (object): Argument.
         asserted_type (type, optional): Type of result.
     """
+    if ref:
+        op_ref = ref
+    else:
+        op_ref = op
+
     x_dense = B.dense(x)
+
     res = op(x)
-    approx(res, op(x_dense))
+    res_ref = op_ref(x_dense)
+    approx(res, res_ref)
+
     _assert_instance(res, asserted_type)
 
 
-def check_bin_op(op, x, y, asserted_type=object, check_broadcasting=True):
+def check_bin_op(
+    op,
+    x,
+    y,
+    ref=None,
+    asserted_type=object,
+    check_broadcasting=True,
+    align_dense_batch=False,
+):
     """Assert the correct of a binary operation by checking whether the
     result is the same on the dense versions of the arguments.
 
@@ -130,20 +152,35 @@ def check_bin_op(op, x, y, asserted_type=object, check_broadcasting=True):
         op (function): Binary operation to check.
         x (object): First argument.
         y (object): Second argument.
+        ref (function, optional): Set this as the function to be used as the reference
+            to which the dense matrices will be fed.
         asserted_type (type, optional): Type of result.
         check_broadcasting (bool, optional): Check broadcasting behaviour.
+        align_dense_batch (bool, optional): Broadcast the batch dimensions of the
+            dense matrices to check against. This can be used if the operation for
+            dense matrices does not support broadcasting of batch dimensions.
     """
+    if ref:
+        op_ref = ref
+    else:
+        op_ref = op
+
     x_dense = B.dense(x)
     y_dense = B.dense(y)
-    res = op(x, y)
+    if align_dense_batch:
+        x_dense, y_dense = align_batch(x_dense, y_dense)
 
-    approx(res, op(x_dense, y_dense))
+    res = op(x, y)
+    res_ref = op_ref(x_dense, y_dense)
+    approx(res, res_ref)
+
+    # Test any possible caching.
+    approx(op(x, y), op_ref(x_dense, y_dense))
 
     with IgnoreDenseWarning():
         if check_broadcasting:
-            approx(op(x_dense, y), op(x_dense, y_dense))
-            approx(op(x, y_dense), op(x_dense, y_dense))
-        approx(op(x_dense, y_dense), op(x_dense, y_dense))
+            approx(op(x_dense, y), res_ref)
+            approx(op(x, y_dense), res_ref)
 
     _assert_instance(res, asserted_type)
 
@@ -233,6 +270,16 @@ def generate(code):
     Returns:
         tensor: Random tensor.
     """
+    # Handle batch shape:
+    batch_code, code = code.split("|")
+
+    # Parse batch.
+    if batch_code == "":
+        batch = ()
+    else:
+        batch = tuple(int(d) for d in batch_code.split(","))
+
+    # Handle matrix shape:
     mat_code, shape_code = code.split(":")
 
     # Parse shape.
@@ -242,47 +289,47 @@ def generate(code):
         shape = tuple(int(d) for d in shape_code.split(","))
 
     if mat_code == "randn":
-        return B.randn(*shape)
+        return B.randn(*batch, *shape)
     elif mat_code == "randn_pd":
-        mat = B.randn(*shape)
+        mat = B.randn(*batch, *shape)
 
         # If it is a scalar or vector, just pointwise square it.
         if len(shape) in {0, 1}:
             return mat ** 2 + 1
         else:
-            return B.matmul(mat, mat, tr_b=True) + B.eye(shape[0])
+            return B.matmul(mat, mat, tr_b=True) + B.eye(*batch, shape[0], shape[0])
 
     elif mat_code == "zero":
-        return Zero(B.default_dtype, *shape)
+        return Zero(B.default_dtype, *batch, *shape)
 
     elif mat_code == "const":
-        return Constant(B.randn(), *shape)
+        return Constant(B.randn(*batch), *shape)
     elif mat_code == "const_pd":
-        return Constant(B.randn() ** 2 + 1, *shape)
+        return Constant(B.randn(*batch) ** 2 + 1, *shape)
 
     elif mat_code == "lt":
-        mat = B.vec_to_tril(B.randn(int(0.5 * shape[0] * (shape[0] + 1))))
+        mat = B.vec_to_tril(B.randn(*batch, int(0.5 * shape[0] * (shape[0] + 1))))
         return LowerTriangular(mat)
     elif mat_code == "lt_pd":
-        mat = generate(f"randn_pd:{shape[0]},{shape[0]}")
+        mat = generate(f"{batch_code}|randn_pd:{shape[0]},{shape[0]}")
         return LowerTriangular(B.cholesky(mat))
 
     elif mat_code == "ut":
-        mat = B.vec_to_tril(B.randn(int(0.5 * shape[0] * (shape[0] + 1))))
+        mat = B.vec_to_tril(B.randn(*batch, int(0.5 * shape[0] * (shape[0] + 1))))
         return UpperTriangular(B.transpose(mat))
     elif mat_code == "ut_pd":
-        mat = generate(f"randn_pd:{shape[0]},{shape[0]}")
+        mat = generate(f"{batch_code}|randn_pd:{shape[0]},{shape[0]}")
         return UpperTriangular(B.transpose(B.cholesky(mat)))
 
     elif mat_code == "dense":
-        return Dense(generate(f"randn:{shape_code}"))
+        return Dense(generate(f"{batch_code}|randn:{shape_code}"))
     elif mat_code == "dense_pd":
-        return Dense(generate(f"randn_pd:{shape_code}"))
+        return Dense(generate(f"{batch_code}|randn_pd:{shape_code}"))
 
     elif mat_code == "diag":
-        return Diagonal(generate(f"randn:{shape_code}"))
+        return Diagonal(generate(f"{batch_code}|randn:{shape_code}"))
     elif mat_code == "diag_pd":
-        return Diagonal(generate(f"randn_pd:{shape_code}"))
+        return Diagonal(generate(f"{batch_code}|randn_pd:{shape_code}"))
 
     else:
         raise RuntimeError(f'Cannot parse generation code "{code}".')
@@ -301,159 +348,191 @@ concat_warnings = [
 
 # Fixtures:
 
-
-@pytest.fixture()
-def mat1():
-    return generate("randn:6,6")
+batch_prefixes = ["|", "3|"]
 
 
-@pytest.fixture()
-def mat2(mat1):
-    return generate("randn:6,6")
+@pytest.fixture(params=[0, 1, 2])
+def gen_batch_code(request):
+    if request.param == 0:
+        # No batches
+        return lambda: "|"
+    elif request.param == 1:
+        # All batched
+        return lambda: "3|"
+    else:
+        # Random mixture
+        return lambda: np.random.choice(batch_prefixes)
 
 
-@pytest.fixture()
-def vec1():
-    return generate("randn:6")
+@_dispatch
+def loop_batches(code: str):
+    return [prefix + code for prefix in batch_prefixes]
 
 
-@pytest.fixture()
-def vec2():
-    return generate("randn:6")
+@_dispatch
+def loop_batches(codes: tuple):
+    return list(product(*(loop_batches(code) for code in codes)))
 
 
-@pytest.fixture()
-def scalar1():
-    return generate("randn:")
+@_dispatch
+def loop_batches(codes: list):
+    return [prefixed_code for code in codes for prefixed_code in loop_batches(code)]
 
 
-@pytest.fixture()
-def scalar2():
-    return generate("randn:")
+@pytest.fixture(params=loop_batches("randn:6,6"))
+def mat1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def zero1():
-    return generate("zero:6,6")
+@pytest.fixture(params=loop_batches("randn:6,6"))
+def mat2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def zero2():
-    return generate("zero:6,6")
+@pytest.fixture(params=loop_batches("randn:6"))
+def vec1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def zero_r():
-    return generate("zero:6,4")
+@pytest.fixture(params=loop_batches("randn:6"))
+def vec2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def dense1():
-    return generate("dense:6,6")
+@pytest.fixture(params=loop_batches("randn:"))
+def scalar1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def dense2():
-    return generate("dense:6,6")
+@pytest.fixture(params=loop_batches("randn:"))
+def scalar2(request):
+    return generate(request.param)
 
 
-@pytest.fixture(params=["dense:6,1", "dense:1,6", "dense:6,6"])
+@pytest.fixture(params=loop_batches("zero:6,6"))
+def zero1(request):
+    return generate(request.param)
+
+
+@pytest.fixture(params=loop_batches("zero:6,6"))
+def zero2(request):
+    return generate(request.param)
+
+
+@pytest.fixture(params=loop_batches("zero:6,4"))
+def zero_r(request):
+    return generate(request.param)
+
+
+@pytest.fixture(params=loop_batches("dense:6,6"))
+def dense1(request):
+    return generate(request.param)
+
+
+@pytest.fixture(params=loop_batches("dense:6,6"))
+def dense2(request):
+    return generate(request.param)
+
+
+@pytest.fixture(params=loop_batches(["dense:6,1", "dense:1,6", "dense:6,6"]))
 def dense_bc(request):
     return generate(request.param)
 
 
-@pytest.fixture()
-def dense_r():
-    return generate("dense:6,4")
+@pytest.fixture(params=loop_batches("dense:6,4"))
+def dense_r(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def dense_pd():
-    return generate("dense_pd:6,6")
+@pytest.fixture(params=loop_batches("dense_pd:6,6"))
+def dense_pd(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def diag1():
-    return generate("diag:6")
+@pytest.fixture(params=loop_batches("diag:6"))
+def diag1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def diag2():
-    return generate("diag:6")
+@pytest.fixture(params=loop_batches("diag:6"))
+def diag2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def diag_pd():
-    return generate("diag_pd:6")
+@pytest.fixture(params=loop_batches("diag_pd:6"))
+def diag_pd(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def const1():
-    return generate("const:6,6")
+@pytest.fixture(params=loop_batches("const:6,6"))
+def const1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def const2():
-    return generate("const:6,6")
+@pytest.fixture(params=loop_batches("const:6,6"))
+def const2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def const_r():
-    return generate("const:6,4")
+@pytest.fixture(params=loop_batches("const:6,4"))
+def const_r(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def const_pd():
-    return generate("const_pd:6,6")
+@pytest.fixture(params=loop_batches("const_pd:6,6"))
+def const_pd(request):
+    return generate(request.param)
 
 
-@pytest.fixture(params=["randn:", "const:6,6"])
+@pytest.fixture(params=["|randn:"] + loop_batches(["const:6,6"]))
 def const_or_scalar1(request):
     return generate(request.param)
 
 
-@pytest.fixture(params=["randn:", "const:6,6"])
+@pytest.fixture(params=["|randn:"] + loop_batches(["const:6,6"]))
 def const_or_scalar2(request):
     return generate(request.param)
 
 
-@pytest.fixture()
-def lt1():
-    return generate("lt:6")
+@pytest.fixture(params=loop_batches("lt:6"))
+def lt1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def lt2():
-    return generate("lt:6")
+@pytest.fixture(params=loop_batches("lt:6"))
+def lt2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def lt_pd():
-    return generate("lt_pd:6")
+@pytest.fixture(params=loop_batches("lt_pd:6"))
+def lt_pd(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def ut1():
-    return generate("ut:6")
+@pytest.fixture(params=loop_batches("ut:6"))
+def ut1(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def ut2():
-    return generate("ut:6")
+@pytest.fixture(params=loop_batches("ut:6"))
+def ut2(request):
+    return generate(request.param)
 
 
-@pytest.fixture()
-def ut_pd():
-    return generate("ut_pd:6")
+@pytest.fixture(params=loop_batches("ut_pd:6"))
+def ut_pd(request):
+    return generate(request.param)
 
 
 @pytest.fixture(
-    params=[
-        ("dense:6,1", "dense:1,1"),
-        ("dense:6,2", "dense:2,2"),
-        ("dense:6,3", "dense:3,3"),
-        ("diag:6", "dense:6,6"),
-    ]
+    params=loop_batches(
+        [
+            ("dense:6,1", "dense:1,1"),
+            ("dense:6,2", "dense:2,2"),
+            ("dense:6,3", "dense:3,3"),
+            ("diag:6", "dense:6,6"),
+        ]
+    )
 )
 def lr1(request):
     code_lr, code_m = request.param
@@ -461,12 +540,14 @@ def lr1(request):
 
 
 @pytest.fixture(
-    params=[
-        ("dense:6,1", "dense:1,1"),
-        ("dense:6,2", "dense:2,2"),
-        ("dense:6,3", "dense:3,3"),
-        ("diag:6", "dense:6,6"),
-    ]
+    params=loop_batches(
+        [
+            ("dense:6,1", "dense:1,1"),
+            ("dense:6,2", "dense:2,2"),
+            ("dense:6,3", "dense:3,3"),
+            ("diag:6", "dense:6,6"),
+        ]
+    )
 )
 def lr2(request):
     code_lr, code_m = request.param
@@ -474,12 +555,14 @@ def lr2(request):
 
 
 @pytest.fixture(
-    params=[
-        ("dense:6,1", "dense:4,1", "dense:1,1"),
-        ("dense:6,2", "dense:4,2", "dense:2,2"),
-        ("dense:6,3", "dense:4,3", "dense:3,3"),
-        ("diag:6", "dense:4,6", "dense:6,6"),
-    ]
+    params=loop_batches(
+        [
+            ("dense:6,1", "dense:4,1", "dense:1,1"),
+            ("dense:6,2", "dense:4,2", "dense:2,2"),
+            ("dense:6,3", "dense:4,3", "dense:3,3"),
+            ("diag:6", "dense:4,6", "dense:6,6"),
+        ]
+    )
 )
 def lr_r(request):
     code_l, code_r, code_m = request.param
@@ -487,12 +570,14 @@ def lr_r(request):
 
 
 @pytest.fixture(
-    params=[
-        ("dense:6,1", "dense_pd:1,2"),
-        ("dense:6,2", "dense_pd:2,2"),
-        ("dense:6,3", "dense_pd:3,3"),
-        ("diag:6", "diag_pd:6"),
-    ]
+    params=loop_batches(
+        [
+            ("dense:6,1", "dense_pd:1,2"),
+            ("dense:6,2", "dense_pd:2,2"),
+            ("dense:6,3", "dense_pd:3,3"),
+            ("diag:6", "diag_pd:6"),
+        ]
+    )
 )
 def lr_pd(request):
     code_l, code_m = request.param
@@ -515,16 +600,18 @@ def wb_pd(diag_pd, lr_pd):
 
 
 @pytest.fixture(
-    params=[
-        ("diag:1", "diag:6"),
-        ("diag:2", "diag:3"),
-        ("diag:3", "diag:2"),
-        ("diag:6", "diag:1"),
-        ("dense:1,1", "dense:6,6"),
-        ("dense:2,2", "dense:3,3"),
-        ("dense:3,3", "dense:2,2"),
-        ("dense:6,6", "dense:1,1"),
-    ]
+    params=loop_batches(
+        [
+            ("diag:1", "diag:6"),
+            ("diag:2", "diag:3"),
+            ("diag:3", "diag:2"),
+            ("diag:6", "diag:1"),
+            ("dense:1,1", "dense:6,6"),
+            ("dense:2,2", "dense:3,3"),
+            ("dense:3,3", "dense:2,2"),
+            ("dense:6,6", "dense:1,1"),
+        ]
+    )
 )
 def kron1(request):
     code1, code2 = request.param
@@ -532,16 +619,18 @@ def kron1(request):
 
 
 @pytest.fixture(
-    params=[
-        ("diag:1", "diag:6"),
-        ("diag:2", "diag:3"),
-        ("diag:3", "diag:2"),
-        ("diag:6", "diag:1"),
-        ("dense:1,1", "dense:1,1"),
-        ("dense:2,2", "dense:3,3"),
-        ("dense:3,3", "dense:2,2"),
-        ("dense:6,6", "dense:1,1"),
-    ]
+    params=loop_batches(
+        [
+            ("diag:1", "diag:6"),
+            ("diag:2", "diag:3"),
+            ("diag:3", "diag:2"),
+            ("diag:6", "diag:1"),
+            ("dense:1,1", "dense:1,1"),
+            ("dense:2,2", "dense:3,3"),
+            ("dense:3,3", "dense:2,2"),
+            ("dense:6,6", "dense:1,1"),
+        ]
+    )
 )
 def kron2(request):
     code1, code2 = request.param
@@ -549,12 +638,14 @@ def kron2(request):
 
 
 @pytest.fixture(
-    params=[
-        ("diag:2", "dense:3,2"),
-        ("dense:3,2", "diag:2"),
-        ("dense:1,4", "dense:6,1"),
-        ("dense:6,1", "dense:1,4"),
-    ]
+    params=loop_batches(
+        [
+            ("diag:2", "dense:3,2"),
+            ("dense:3,2", "diag:2"),
+            ("dense:1,4", "dense:6,1"),
+            ("dense:6,1", "dense:1,4"),
+        ]
+    )
 )
 def kron_r(request):
     code1, code2 = request.param
@@ -562,16 +653,18 @@ def kron_r(request):
 
 
 @pytest.fixture(
-    params=[
-        ("diag_pd:1", "diag_pd:6"),
-        ("diag_pd:2", "diag_pd:3"),
-        ("diag_pd:3", "diag_pd:2"),
-        ("diag_pd:6", "diag_pd:1"),
-        ("dense_pd:1,1", "dense_pd:6,6"),
-        ("dense_pd:2,2", "dense_pd:3,3"),
-        ("dense_pd:3,3", "dense_pd:2,2"),
-        ("dense_pd:6,6", "dense_pd:1,1"),
-    ]
+    params=loop_batches(
+        [
+            ("diag_pd:1", "diag_pd:6"),
+            ("diag_pd:2", "diag_pd:3"),
+            ("diag_pd:3", "diag_pd:2"),
+            ("diag_pd:6", "diag_pd:1"),
+            ("dense_pd:1,1", "dense_pd:6,6"),
+            ("dense_pd:2,2", "dense_pd:3,3"),
+            ("dense_pd:3,3", "dense_pd:2,2"),
+            ("dense_pd:6,6", "dense_pd:1,1"),
+        ]
+    )
 )
 def kron_pd(request):
     code1, code2 = request.param
@@ -579,35 +672,46 @@ def kron_pd(request):
 
 
 @pytest.fixture(
-    params=[
-        ("diag:1", "diag:6"),
-        ("diag:2", "diag:3"),
-        ("diag:3", "diag:2"),
-        ("diag:6", "diag:1"),
-        ("dense:1,6", "dense:6,1"),
-        ("dense:2,3", "dense:3,2"),
-        ("dense:3,2", "dense:2,3"),
-        ("dense:6,1", "dense:1,6"),
-    ]
+    params=loop_batches(
+        [
+            ("diag:1", "diag:6"),
+            ("diag:2", "diag:3"),
+            ("diag:3", "diag:2"),
+            ("diag:6", "diag:1"),
+            ("dense:1,6", "dense:6,1"),
+            ("dense:2,3", "dense:3,2"),
+            ("dense:3,2", "dense:2,3"),
+            ("dense:6,1", "dense:1,6"),
+        ]
+    )
 )
 def kron_mixed(request):
     code1, code2 = request.param
     return Kronecker(generate(code1), generate(code2))
 
 
-@pytest.fixture(params=[0, 1])
+@pytest.fixture(params=[-2, -1, 0, 1, 2])
 def tb_axis(request):
     return request.param
 
 
 _tb_codes = [
-    [("dense:3,3", 2), ("zero:3,3", 3), ("lt:3,3", 4)],
-    [("diag:3", 2), ("ut:3,3", 3), ("diag:3", 1)],
+    [(c1, 2), (c2, 3), (c3, 4)]
+    for c1, c2, c3 in loop_batches([("dense:3,3", "zero:3,3", "lt:3,3")])
+] + [
+    [(c1, 2), (c2, 3), (c3, 1)]
+    for c1, c2, c3 in loop_batches([("diag:3", "ut:3,3", "diag:3")])
 ]
 
 
 @pytest.fixture(params=_tb_codes)
 def tb1(request, tb_axis):
+    # Ensure that the axis is compatible.
+    if tb_axis in {-2, 2} and all(code.startswith("|") for code, _ in request.param):
+        if tb_axis == -2:
+            tb_axis = -1
+        else:
+            tb_axis = 1
     return TiledBlocks(
         *((generate(code), rep) for code, rep in request.param), axis=tb_axis
     )
@@ -615,6 +719,12 @@ def tb1(request, tb_axis):
 
 @pytest.fixture(params=_tb_codes)
 def tb2(request, tb_axis):
+    # Ensure that the axis is compatible.
+    if tb_axis in {-2, 2} and all(code.startswith("|") for code, _ in request.param):
+        if tb_axis == -2:
+            tb_axis = -1
+        else:
+            tb_axis = 1
     return TiledBlocks(
         *((generate(code), rep) for code, rep in request.param), axis=tb_axis
     )
